﻿using NUnit.Framework;
using System.Diagnostics;

namespace tcharttest
{
    /// <summary>
    /// DoubleExtensionsTest
    /// </summary>
    [TestFixture]
    public class DoubleExtensionsTest
    {
        /// <summary>
        /// Nexts up test.
        /// </summary>
        /// <param name="val">The val.</param>
        [TestCase(-1.0)]
        [TestCase(0.0)]
        [TestCase(1.0)]
        [TestCase(-1.0e+014)]
        [TestCase(-1.0e+015)]
        [TestCase(-1.0e+016)]
        [TestCase(1.0e+014)]
        [TestCase(1.0e+015)]
        [TestCase(1.0e+016)]
        [TestCase(double.NaN)]
        [TestCase(double.Epsilon)]
        [TestCase(double.MinValue)]
        [TestCase(double.MaxValue)]
        [TestCase(double.NegativeInfinity)]
        [TestCase(double.PositiveInfinity)]
        public void NextUpTest(double val)
        {
            double result = val.NextUp();

            if (val == double.MaxValue)
                Assert.AreEqual(double.MaxValue, result);
            else if (val == double.NegativeInfinity)
                Assert.AreEqual(double.NegativeInfinity, result);
            else if (val == double.PositiveInfinity)
                Assert.AreEqual(double.PositiveInfinity, result);
            else if (double.IsNaN(val))
                Assert.AreEqual(double.NaN, result);
            else
                Assert.Greater(result, val);
        }

        /// <summary>
        /// Nexts down test.
        /// </summary>
        /// <param name="val">The val.</param>
        [TestCase(-1.0)]
        [TestCase(0.0)]
        [TestCase(1.0)]
        [TestCase(-1.0e+014)]
        [TestCase(-1.0e+015)]
        [TestCase(-1.0e+016)]
        [TestCase(1.0e+014)]
        [TestCase(1.0e+015)]
        [TestCase(1.0e+016)]
        [TestCase(double.NaN)]
        [TestCase(double.Epsilon)]
        [TestCase(double.MinValue)]
        [TestCase(double.MaxValue)]
        [TestCase(double.NegativeInfinity)]
        [TestCase(double.PositiveInfinity)]
        public void NextDownTest(double val)
        {
            double result = val.NextDown();

            if (val == double.MinValue)
                Assert.AreEqual(double.MinValue, result);
            else if (val == double.NegativeInfinity)
                Assert.AreEqual(double.NegativeInfinity, result);
            else if (val == double.PositiveInfinity)
                Assert.AreEqual(double.PositiveInfinity, result);
            else if (double.IsNaN(val))
                Assert.AreEqual(double.NaN, result);
            else
                Assert.Less(result, val);
        }

        /// <summary>
        /// Changes the by ulp test.
        /// </summary>
        /// <param name="val">The val.</param>
        [TestCase(-1.0)]
        [TestCase(0.0)]
        [TestCase(1.0)]
        [TestCase(-1.0e+014)]
        [TestCase(-1.0e+015)]
        [TestCase(-1.0e+016)]
        [TestCase(1.0e+014)]
        [TestCase(1.0e+015)]
        [TestCase(1.0e+016)]
        [TestCase(double.NaN)]
        [TestCase(double.Epsilon)]
        [TestCase(double.MinValue)]
        [TestCase(double.MaxValue)]
        [TestCase(double.NegativeInfinity)]
        [TestCase(double.PositiveInfinity)]
        public void ChangeByUlpTest(double val)
        {
            double result = val.ChangeByUlp(1);

            if (val == double.NegativeInfinity)
                Assert.AreEqual(double.NegativeInfinity, result);
            else if (val == double.PositiveInfinity)
                Assert.AreEqual(double.PositiveInfinity, result);
            else if (double.IsNaN(val))
                Assert.AreEqual(double.NaN, result);
            else if (val == double.MaxValue)
                Assert.AreEqual(double.PositiveInfinity, result);
            else
                Assert.Greater(result, val);


            result = val.ChangeByUlp(-1);

            if (val == double.NegativeInfinity)
                Assert.AreEqual(double.NegativeInfinity, result);
            else if (val == double.PositiveInfinity)
                Assert.AreEqual(double.PositiveInfinity, result);
            else if (double.IsNaN(val))
                Assert.AreEqual(double.NaN, result);
            else if (val == double.MinValue)
                Assert.AreEqual(double.NegativeInfinity, result);
            else
                Assert.Less(result, val);
        }

        /// <summary>
        /// Gets the ulp test.
        /// </summary>
        [Test]
        public void GetUlpTest()
        {
            double val1 = 0.00001;
            double val2 = 1.0;
            double val3 = 1.0e10;

            Assert.Greater(val2.GetUlp(), val1.GetUlp());
            Assert.Greater(val3.GetUlp(), val2.GetUlp());
        }

        /// <summary>
        /// Check function SignificantDifference test.
        /// </summary>
        [Test]
        public void SignificantDifferenceTest()
        {
            const double d1 = 1.0e020;  const double d3 = 1.00000000000000001e020;
                                        const double d4 = 1.0000000000000001e020;
            const double d2 = 1.0e-020; const double d5 = 1.00000000000000001e-020;
                                        const double d6 = 1.0000000000000001e-020;

            Trace.WriteLine( d1.SignificantDifference( d3 ) + " : " + d1.ToExactString() + " - " + d3.ToExactString() ); // false: ( 1.0e020  - 1.00000000000000001e020  )
            Assert.IsFalse ( d1.SignificantDifference( d3 ), "d1 shouldn't have a significant difference compared to d3" );
            Trace.WriteLine( d1.SignificantDifference( d4 ) + "  : " + d1.ToExactString() + " - " + d4.ToExactString() );// true:  ( 1.0e020  - 1.0000000000000001e020   )
            Assert.IsTrue  ( d1.SignificantDifference( d4 ), "d1 should have a significant difference compared to d4" );
            Trace.WriteLine( d2.SignificantDifference( d5 ) + " : " + d2.ToExactString() + " - " + d5.ToExactString() ); // false: ( 1.0e-020 - 1.00000000000000001e-020 )
            Assert.IsFalse ( d2.SignificantDifference( d5 ), "d2 shouldn't have a significant difference compared to d5" );
            Trace.WriteLine( d2.SignificantDifference( d6 ) + "  : " + d2.ToExactString() + " - " + d6.ToExactString() );// true:  ( 1.0e-020 - 1.0000000000000001e-020  )
            Assert.IsTrue  ( d2.SignificantDifference( d6 ), "d2 should have a significant difference compared to d6" );
        }
    }
}